#!/bin/bash
# create a zip file from a transcript, containing the new files. wmertens@cisco.com
#
TRAN="$1"
shift
zipfile="$1"
shift

T=/var/radmind/tmp/pca.$$
DLDIR=$T.dl
TOGET=$T.T

clean() {
	if [ -d $DLDIR ]; then
		echo "$DLDIR exists, please clean it!"
	fi
}
usage() {
	[ -n "$*" ] && echo "$*
Usage: $0 <transcript name> <zipfile name> <lapply arguments>"
	clean
	exit 1
}
die() {
	[ -n "$*" ] && echo "ERROR: $*"
	clean
	exit 1
}

trap clean 1 2 3 9 15

[ -r "$TRAN" ] || usage "Cannot read transcript $TRAN!"
touch "$zipfile" || usage "Cannot write to $zipfile!"
rm "$zipfile"
case "$zipfile" in *.zip) : ;; *) zipfile="$zipfile.zip" ;; esac

echo Processing transcript

# Get the downloadable part, create directories where needed, relativize
perl -nle '
if (/^[^ ]+:/) {
	$transcript = $_;
}
if (/^\+/) {
	@parts = split;
	if ( $parts[2] =~ /^\// ) {
		s/\//.\//;
	}
	$parts[2] =~ s/^\.//;
	$parts[2] =~ s/^\///;
	@now = split "/", $parts[2];
	$i=0;
	while ($i < $#last && $i < $#now) {
		last if $last[$i] ne $now[$i];
		$i++;
	}
	while ($i < $#now) {
		print "d ./".join("/",@now[0 .. $i])." 700 0 0";
		$i++;
	}
	print $transcript if $transcript ne $lasttrans;
	$lasttrans = $transcript;
	print $_;
	@last=@now;
}' "$TRAN" > $TOGET

# Download fileset. This is the only bit that needs the network
echo Downloading files
mkdir $DLDIR
cd $DLDIR
lapply "$@" $TOGET || die "download failed"
cd -

# Create archive
echo Creating archive $zipfile
ditto -c -k --sequesterRsrc "$DLDIR" "$zipfile" || die "Could not create $zipfile"
rm -rf "$DLDIR"
ls -l "$zipfile"
echo "Done. Extract the archive with"
echo "  # ditto -x -k --sequesterRsrc $zipfile /"
